//====================================================================================================
// ExtWindow.cpp
//====================================================================================================

#include "StdAfx.h"
#include "ExtWindow.h"
#include "MDIBackGround.h"

extern HINSTANCE g_hInstance;

//====================================================================================================
// CExtWindow::CExtWindow()
//====================================================================================================
CExtWindow::CExtWindow(LPCWSTR lpszClassName) : CBackGround(), m_hExtWnd(NULL) {  // HINSTANCE hInstance, 
	ATLASSERT(g_hInstance != NULL);

	m_lpszClassName = lpszClassName;  // L"ExternalWindow"
	ATLASSERT(m_lpszClassName != NULL);

	RegisterClass(lpszClassName);  // g_hInstance, 
}  // CExtWindow::CExtWindow()

//====================================================================================================
// CExtWindow::~CExtWindow()
//====================================================================================================
CExtWindow::~CExtWindow() {
	UnRegisterClass();
}  // CExtWindow::~CExtWindow()

//====================================================================================================
// CExtWindow::RegisterClass()
//====================================================================================================
ATOM CExtWindow::RegisterClass(LPCWSTR lpszClassName) {  // HINSTANCE hInstance, 
	WNDCLASSEX wc = {0};
		
	// Registering the Window Class
	wc.cbSize = sizeof(WNDCLASSEX);
	wc.style = 0;
	wc.lpfnWndProc = WindowProc;
	wc.cbClsExtra = 0;
	wc.cbWndExtra = 0;
	wc.hInstance = g_hInstance;
	wc.hIcon = LoadIcon(NULL, IDI_APPLICATION);
	wc.hCursor = LoadCursor(NULL, IDC_ARROW);
	wc.hbrBackground = (HBRUSH) (COLOR_WINDOW + 1);
	wc.lpszMenuName = NULL;
	wc.lpszClassName = lpszClassName;
	wc.hIconSm = LoadIcon(NULL, IDI_APPLICATION);

	ATOM Atom = ::RegisterClassEx(&wc);
	ATLASSERT(Atom != 0);

	return Atom; 
}  // CExtWindow::RegisterClass()

//====================================================================================================
// CExtWindow::CreateEx() -   ,      
//====================================================================================================
HWND CExtWindow::CreateEx(HWND hWndParent, LPCWSTR lpszWindowName, RECT &Rect, DWORD dwExStyle, 
		DWORD dwStyle, HMENU hMenu, LPVOID lpCreateParam) {

	//----------------------------------------------------------------------------------------------------
	//   
	//----------------------------------------------------------------------------------------------------

	HWND hExtWnd = CreateWindowEx(
		dwExStyle,
		m_lpszClassName,
		lpszWindowName,
		dwStyle,
		Rect.left, Rect.top, Rect.right, Rect.bottom,
		NULL,  // hWndParent, 
		hMenu, 
		g_hInstance,
		lpCreateParam
	);
		
	if(!hExtWnd) {
		MessageBox(NULL, L"Window Creation Failed!", L"Error!", MB_ICONEXCLAMATION | MB_OK);
		return NULL;
	}  // if(!hExtWnd)
	
	m_hExtWnd = hExtWnd;

	//----------------------------------------------------------------------------------------------------
	//         
	//----------------------------------------------------------------------------------------------------

	if(!LoadBitmap()) {
		MessageBox(NULL, L"LoadBitmap() == NULL!", L"Error", MB_OK);
		return NULL;
	}  // if(LoadBitmap())

	//----------------------------------------------------------------------------------------------------
	//  
	//----------------------------------------------------------------------------------------------------

	hMenu = CreateMenu();  //  
  HMENU hFileSubMenu = CreatePopupMenu();  //  ""

	AppendMenu(hFileSubMenu, MF_STRING, ID_FILE_IMAGE, L" \tAlt+I");
	AppendMenu(hFileSubMenu, MF_SEPARATOR, 0, NULL);
	AppendMenu(hFileSubMenu, MF_STRING, ID_FILE_EXIT, L"\tCtrl+F4");
		
  HMENU hHelpSubMenu = CreatePopupMenu();  //  ""

	AppendMenu(hHelpSubMenu, MF_STRING, ID_HELP_ABOUT, L"\tAlt+H");
	
	AppendMenu(hMenu, MF_POPUP, (UINT_PTR) hFileSubMenu, L"");
	AppendMenu(hMenu, MF_POPUP, (UINT_PTR) hHelpSubMenu, L"");

	//  
	SetMenu(hExtWnd, hMenu);

	//----------------------------------------------------------------------------------------------------
	//   
	//----------------------------------------------------------------------------------------------------

	RECT R = {0};	
	
	//      
	GetClientRect(hExtWnd, &R);
	//BOOL bResult = SystemParametersInfo(SPI_GETWORKAREA, 0, &R, 0);

	//    .        !
	SetParent(hExtWnd, hWndParent);

	//    (  -   )		
	ShowWindow(hExtWnd, SW_HIDE);
	ShowWindow(hExtWnd, SW_SHOW);
	
	return hExtWnd;
}  // CExtWindow::CreateEx()

//====================================================================================================
// WindowProc() - Window Procedure
//====================================================================================================
LRESULT CALLBACK CExtWindow::WindowProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam) {
  switch(uMsg) {
		case WM_ERASEBKGND:
			//        
      return 0;
		case WM_COMMAND: {
			//     .
			switch(LOWORD(wParam)) {
				case ID_FILE_IMAGE:
					OpenImage(hWnd);  //     
					return 0;
				case ID_FILE_EXIT:
					//   
					CloseExtWindow(hWnd);			
					return 0;
				case ID_HELP_ABOUT:
					AboutDll(hWnd);
					return 0;
			}  // switch(LOWORD(wParam))

			break;
		}  // case WM_COMMAND
		case WM_KEYDOWN: {
			switch(wParam) {
				case VK_ESCAPE:
					CloseExtWindow(hWnd);  //  ()    Esc
					return 0;
				case VK_F4: {
					bool bCtrl = ((::GetAsyncKeyState(VK_CONTROL) & 0x8000) != 0)	? true : false;
					
					if(bCtrl) {
						CloseExtWindow(hWnd);  //  ()    Ctrl+F4
						return 0;
					}  // if(bCtrl)
					
					break;
				}  // case VK_F4
			}  // switch(wParam)
		}  // case WM_KEYDOWN
    case WM_SYSCHAR:  //         Alt + Char
    case WM_MENUCHAR: {  //         Alt + Char
      if(wParam == 'i' || wParam == 'I' || wParam == 0x448 || wParam == 0x428) {  // 0x448 = ''; 0x428 = ''
				OpenImage(hWnd);  //     
				return 0;
      }  // if(wParam == 'i' || wParam == 'I' || wParam == 0x448 || wParam == 0x428)

			else if(wParam == 'l' || wParam == 'L' || wParam == 0x434 || wParam == 0x414) {  // 0x434 = ''; 0x414 = ''
				AboutDll(hWnd);
				return 0;
      }  // else if(wParam == 'l' || wParam == 'L' || wParam == 0x434 || wParam == 0x414)

			break;
    }  // case WM_MENUCHAR:
    case WM_PAINT:
			DoPaint(hWnd);
			return 0;
		case WM_CLOSE:
			//   
			CloseExtWindow(hWnd);  //  ()      ""
			return 0;
		default:
			break;
  }  // switch(uMsg)

	return DefWindowProc(hWnd, uMsg, wParam, lParam);
}  // WindowProc()

//====================================================================================================
//====================================================================================================
